/* Значимость элементов восприятия - как объекта произвольного внимания:
того из всего воспринимаемого, что имеет наибольшую значимость
т.к. именно наибольшая значимость должна осмысливаться.

Кроме того, значимости объектов - это и есть модель понимания данного объекта внимания -
его значимость в разных условиях и то, какие действия могут быть совершены при этом.

Сохраняется в файле importance.txt в формате ID|ProblemID|Type|ObjectID|Value

Значимости в коде проекта (func getObjectImportance) обычно имеют значения величин от -10 0 до 10,
но в случае значимости эффекта от реагирования от -1 0 до 1,
а в случаях усредненной значимости компонентов объекта Ответа
(акты воздействия, фразы и отдельные слова, принимающие значимость фразы)
не органичены в минус и плюс. В последнем случае такая значимость берется из Эффекта пробных действий
и связывается ос всеми компонентами воспринимаемого в этих условиях функцией setImportance().
Значимость всегда определяется в контексте всех предшествующих условий,
т.е. специфична для активностей деревьев автоматизмов и понимания.

При каждом вызове consciousnessElementary определяется текущий объект наибольшой значимости в воспринимаемом -
в функции определения текущей Цели getMentalPurpose().

!!!Если уже была определена иная значимость ранее для объекта внимания в условиях ProblemID, то создается новый образ
так что можно получить обобщенное значение значимости методом простого усреднения или учитывать возможные граничные случаи:
func getAllObjectsImportance


по extremImportance найти все []*importance: getImportanceFromExtremImportance(objID int, kind int)
*/

package psychic

import (
	"BOT/lib"
	"strconv"
	"strings"
)

/////////////////////////////////////////////////

// типы объектов значимости
var importanceType=[]int{
	1, // ID ActionsImage
	2, // ID MentalActionsImages НЕТ ТАКОГО ТИПА, МЕСТО СВОБОДНО
	3, // ID несловестного действия ActionsImage.ActID[n]
	4, // ID Verbal - при активации дерева автоматизмов
	5, // ID отдельной фразы Verbal.PhraseID[n]
	6,// ID отдельного слова  из Verbal.PhraseID[n]
	7,// ID тон сообщения с Пульта  Verbal.ToneID
	8,// ID настроение оператора  Verbal.MoodID
}
// названия типов объектов значимости
var importanceTypeName=[]string{
	1:"Образ действия",
	2:"образ мысли",
	3:"Моторное действие",
	4:"Вербальный образ",
	5:"Фраза",
	6:"Слово",
	7:"Тон",
	8:"Настроение",
}
///////////////////////////////////////////////

/* Для определения текущих объектов восприятия и выделения одного из них - самого важного
curImportanceObjectArr[] Заполняется в одном месте:
infoFunc18() - "Ощущение CurrentInformationEnvironment - по наиболее значимому параметру"

extremImportance не сохраняется, а возникает только в данной сессиии
Сохраняется extremImportance.extremObjID в importance.txt
*/
type extremImportance struct {
	extremObjID int // ID объекта типа importance
	objID int//  объект значимости - в зависимости от kind м.б. одним из importanceTypeName[] типов
	kind int // тип объекта importanceType
	extremVal int// экстремальная значимость
}
// текущий значимый объект внимания определяется в func consciousnessElementary(
var extremImportanceObject *extremImportance
var extremImportanceObjectOld *extremImportance // против повторения
// текущий значимый объект внимания с отрицательным эффектом extremImportance.extremVal
var problemExtremImportanceObject *extremImportance

// curImportanceObjectArr заполняется в func getGreatestImportance( - Найти наиболее значащие обхъекты в данных условиях
var curImportanceObjectArr []extremImportance //- здесь сохраняются текущие цели внимания к наиболее важному
// для сна
func GetcurImportanceObjectArrLength()int{
	return len(curImportanceObjectArr)
}
// добавить новый extremImportance в curImportanceObjectArr
func addNewcurImportanceObjectArr(e extremImportance){
	for i := 0; i < len(curImportanceObjectArr); i++ {
		if curImportanceObjectArr[i].extremObjID == e.extremObjID {
			curImportanceObjectArr =append(curImportanceObjectArr,e)
		}
	}
}
/* найти extremImportance по extremImportance.extremObjID
curImportanceObjectArr не сохраняется, а возникает только в данной сессиии
Сохраняется extremImportance.extremObjID в importance.txt

id - является идентификатором importance!
 */
func getExtremObjFromID(id int)*extremImportance{
	for i := 0; i < len(curImportanceObjectArr); i++ {
		if curImportanceObjectArr[i].extremObjID == id {
			return &curImportanceObjectArr[i]
		}
	}
	return nil
}
/* вырезать из массива curImportanceObjectArr ссылку на последний важный объект
уменьшить массив на 1
Вырезанный элемент потеряется если его не использовать.
 */
var ePulsCount=0
func cutLastExtremObj()*extremImportance{
	if ePulsCount==PulsCount || curImportanceObjectArr==nil || len(curImportanceObjectArr)==0{
	//	time.Sleep(10 * time.Millisecond)
		return nil
	}
	ePulsCount=PulsCount
	var eObj=curImportanceObjectArr[len(curImportanceObjectArr)-1]
	//уменьшить массив на 1
	curImportanceObjectArr=curImportanceObjectArr[:(len(curImportanceObjectArr)-1)]

return &eObj
}

// вырезать из массива curImportanceObjectArr важный объект после окончания его осмысления
func cutCurrentExtremObj(obj *extremImportance){
	if obj==nil{
		return
	}
	var newArr []extremImportance
	for i := 0; i < len(curImportanceObjectArr); i++ {
if curImportanceObjectArr[i].extremObjID!=obj.extremVal{
	newArr=append(newArr,curImportanceObjectArr[i])
}
		curImportanceObjectArr=newArr
	}
}

// создать экстремальный объект значимости из образа действия actionsImageID
func createExtremImportanceFromActionID(actionsImageID int,extremVal int)*extremImportance {
	id,_:=createNewlastImportanceID(0,detectedActiveLastUnderstandingNodID,1,actionsImageID,extremVal,true)
	var eObj extremImportance
	eObj.extremObjID=id
	eObj.objID=actionsImageID
	eObj.kind=1
	eObj.extremVal=extremVal
	return &eObj
}

//////////////////
// ОКАЗАЛСЯ НЕ НУЖЕН extremImportanceMentalObject хотя есть код с ним
// текущий субъект внимания (объект внимания к собственным мыслям)    ЕЩЕ НЕТ ТАКОГО по аналогии с importance.go
var extremImportanceMentalObject *extremImportance
//////////////////






/////////////////////////////////////////////////////////
// Тот объект, применение которого привело к данной значимости  ID|ProblemID|Type|ObjectID|Value
type importance struct {
	ID int
	// условия точного использования Правила:
	ProblemID int // конечный узел активной ветки дерева проблем detectedActiveLastProblemNodID

	Type int // тип объекта importanceType
	ObjectID int // ID объекта восприянтия
	Value int // значимость от -N 0 до N Чудовищно большие будут говорить о сверх значимости.
}
//var importanceFromID=make(map[int]*importance)
var importanceFromID []*importance  // сам массив
//var AFromID = make([]*aNode, len(strArr))//задать сразу имеющиеся в файле число при загрузке
// запись члена
func WriteImportanceFromID(index int, value *importance) {
	if index >= len(importanceFromID) {
		newSlice := make([]*importance, index+1)
		copy(newSlice, importanceFromID)
		importanceFromID = newSlice
	}
	importanceFromID[index] = value
}
// считывание члена
func ReadeImportanceFromID(index int) (*importance,bool){
	if index >= len(importanceFromID) || importanceFromID[index]==nil {
		return nil,false
	}
	return importanceFromID[index],true
}
///////////////////////////////////////


/* по extremImportance найти все []*importance: getImportanceFromExtremImportance(objID int, kind int)
Все *importance для данного objID и kind

!!! Конкретный объект extremImportance находится по importanceFromID[extremImportance.extremObjID]
 */
func getImportanceFromExtremImportance(objID int, kind int)[]*importance{
	var iArr []*importance
	for _, v := range importanceFromID {
		if v==nil{continue}
		if v.ObjectID==objID && v.Type==kind{
			iArr=append(iArr,v)
		}
	}
return iArr
}
///////////////////////////////////


// выборка по условиям: ProblemID
//индекс карты: ProblemID
//var importanceConditinArr=make(map[int] []*importance)// Массив значимостей при данном условии
var importanceConditinArr [][]*importance  // сам массив
//var AFromID = make([]*aNode, len(strArr))//задать сразу имеющиеся в файле число при загрузке
// запись члена
func WriteImportanceConditinArr(index int, value []*importance) {
	addImportanceConditinArr(index)
	importanceConditinArr[index] = value
}
func addImportanceConditinArr(index int){
	if index >= len(importanceConditinArr) {
		newSlice := make([][]*importance, index+1)
		copy(newSlice, importanceConditinArr)
		importanceConditinArr = newSlice
	}
}
// считывание члена
func ReadeImportanceConditinArr(index int) ([]*importance,bool){
	if index >= len(importanceConditinArr) || importanceConditinArr[index]==nil {
		return nil,false
	}
	return importanceConditinArr[index],true
}
///////////////////////////////////////

/////////////////////////////////////////////////
///////////////////////////////////////////
// вызывается из psychic.go
func ImportanceInit(){
	loadImportance()
}


////////////////////////////////////////////////
// создать новый образ значимости Importance если такого еще нет
var lastImportanceID=0
var isNotImpLoading=true
func createNewlastImportanceID(id int,ProblemID int,Type int,ObjectID int,Value int,CheckUnicum bool)(int,*importance){

	if CheckUnicum {
		oldID,oldVal:=checkUnicumImportance(ProblemID,Type,ObjectID)
		if oldVal!=nil{
			// если есть уже такой объект, то не перекрывать его значимость!
//			oldVal.Value = Value // для возможности произвольно менять значимость и для ее естественного обновления
			if Value !=0{
				oldVal.Value = Value // нельзя обнулять значимость при нейтральном стимуле, иначе не сработает тест: стадия 4, тест 3. Обновляем только если != 0 - что означает явное изменение условий
			}
			return oldID,oldVal
		}
	}

	if id==0{
		lastImportanceID++
		id=lastImportanceID
	}else{
		//		newW.ID=id
		if lastImportanceID<id{
			lastImportanceID=id
		}
	}

	var node importance
	node.ID = id
	node.ProblemID =ProblemID
	node.Type=Type
	node.ObjectID=ObjectID
	node.Value=Value

//	importanceFromID[id]=&node
	WriteImportanceFromID(id, &node)

	//if ProblemID >= len(importanceConditinArr) {
	//}
	// если нет еще индекса ProblemID, то нарастить
	addImportanceConditinArr(ProblemID)
	importanceConditinArr[ProblemID]=append(importanceConditinArr[ProblemID],&node)

	if doWritingFile{
		Saveimportance()
	}

	return id,&node
}
func checkUnicumImportance(ProblemID int,Type int,ObjectID int)(int,*importance){
	for id, v := range importanceFromID {
		if v==nil{continue}
		if ProblemID != v.ProblemID || Type != v.Type || ObjectID != v.ObjectID {
			continue
		}
		return id,v
	}

	return 0,nil
}
/////////////////////////////////////////






//////////////////// сохранить Образы Importance
// ID|ProblemID|Type|ObjectID|Value
func Saveimportance(){
	var out=""
	for k, v := range importanceFromID {
		if v==nil{continue}
		out+=strconv.Itoa(k)+"|"
		out+=strconv.Itoa(v.ProblemID)+"|"
		out+=strconv.Itoa(v.Type)+"|"
		out+=strconv.Itoa(v.ObjectID)+"|"
		out+=strconv.Itoa(v.Value)

		out+="\r\n"
	}
	lib.WriteFileContent(lib.GetMainPathExeFile()+"/memory_psy/importance.txt",out)

}
////////////////////  загрузить образы Importance
// ID|ProblemID|Type|Value
func loadImportance(){
	strArr,_:=lib.ReadLines(lib.GetMainPathExeFile()+"/memory_psy/importance.txt")
	cunt:=len(strArr)
	//importanceConditinArr=make(map[int] []*importance)
	importanceConditinArr = make([][]*importance, cunt)//задать сразу имеющиеся в файле число
	importanceFromID = make([]*importance, cunt)//задать сразу имеющиеся в файле число


	for n := 0; n < cunt; n++ {
		if len(strArr[n])==0{
			continue
		}
		p:=strings.Split(strArr[n], "|")
		id,_:=strconv.Atoi(p[0])
		ProblemID,_:=strconv.Atoi(p[1])
		Type,_:=strconv.Atoi(p[2])
		ObjectID,_:=strconv.Atoi(p[3])
		Value,_:=strconv.Atoi(p[4])

		var saveDoWritingFile= doWritingFile; doWritingFile =false
		isNotImpLoading=false
		createNewlastImportanceID(id,ProblemID,Type,ObjectID,Value,false)
		isNotImpLoading=true
		doWritingFile =saveDoWritingFile
	}
	return

}
/////////////////////////////////////////





///////////////////////////////////////////////////////
/* объект значимости (kind int,ObjectID int) - точно по условиям
Возвращает:
ссылку на объект значимости
*/
func getObjectImportance(kind int,ObjectID int, ProblemID int)(*importance){
	//if ProblemID <= len(importanceConditinArr){
	if len(importanceConditinArr) !=0{
		val,ok:=ReadeImportanceConditinArr(ProblemID)
		if !ok{return nil}
		for _, v := range val {
			if v==nil{continue}
			if v.Type==kind && v.ObjectID ==ObjectID{
				return v
			}
		}
	}

	return nil
}
///////////////////////////////////////////////////////////
/* объект значимости (kind int,ObjectID int) - игнорируя условия detectedActiveLastNodID detectedActiveLastUnderstandingNodID
Возвращает массив найденных объектов
*/
func getObjectImportanceWithoutСonditions(kind int,ObjectID int)([]*importance){
	var outArr []*importance
	for _, v := range importanceFromID {
		if v==nil{continue}
		if v.Type==kind && v.ObjectID ==ObjectID{
			outArr=append(outArr,v)
		}
	}

	return outArr
}
///////////////////////////////////////////////////////////


///////////////////////////////////////////////////////
/* значимость ID объекта внимания (kind int,ObjectID int)
в условиях detectedActiveLastNodID, detectedActiveLastUnderstandingNodID
Возвращает:
1 - ID объекта значимости
2 - значимость
*/
func getObjectsImportanceValue(kind int,ObjectID int, ProblemID int)(int,int){
	if ProblemID < len(importanceConditinArr){
		for _, v := range importanceConditinArr[ProblemID] {
			if v==nil{continue}
			if v.Type==kind && v.ObjectID ==ObjectID{
				return v.ID,v.Value
			}
		}
	}

	return 0,0
}
///////////////////////////////////////////////////////////















